#include <Keypad.h>
#include <SoftwareSerial.h> 
#include <EEPROM.h>
#include <OneWire.h>
#include <DallasTemperature.h>


// Codeschloss by DG1ATN 2022 
// Erweiterung des Beitrages Codeschloss by Tobias Kuch 2020 GPL 3.0
// DG1ATN tobby@indiancoder.de
// Vorlage:
// Repository Ansatz: https://github.com/kuchto
// tobias.kuch@googlemail.com

// #define DEBUG 0

#define LED_G A0    
#define LED_Y A1
#define LED_R A2
#define LED_W A4
#define Photo A6

const byte COLS = 4; //4 Spalten
const byte ROWS = 4; //4 Zeilen

const long presharedKey = 13071978;

char hexaKeys[ROWS][COLS]={
{'D','#','0','*'},
{'C','9','8','7'},
{'B','6','5','4'},
{'A','3','2','1'}
};

byte colPins[COLS] = {2,3,4,5}; //Definition der Pins für die 3 Spalten
byte rowPins[ROWS] = {6,7,8,9}; //Definition der Pins für die 4 Zeilen
char Taste; //Taste ist die Variable für die jeweils gedrückte Taste.
Keypad Tastenfeld = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); //Das Keypad kann absofort mit "Tastenfeld" angesprochen werden

SoftwareSerial com2Main(11,12); // RX, TX

#define ONEwire 10
#define DS18B20_RES 12
DeviceAddress DS18B20_ADR;
OneWire oneWire(ONEwire);
DallasTemperature DS18B20(&oneWire);

union key 
{
    byte as_array[4];
    long as_long;
} initkey;

bool keyPad = true;

int lightOn = 342; //Dämmerungswert
int lightOnMem = lightOn;
int lightOff = 683;

struct duskData
{
    int dark;
    int light;
};
duskData localDust;


void setup()
{
    Serial.begin(9600);
    Serial.println(F("DG1ATN Zahlenschloss Keypad"));
    com2Main.begin(9600);

    // Temeperatursensor einrichten
    DS18B20.begin();
    Serial.print(F("Sesoren: "));
    Serial.println(DS18B20.getDeviceCount(), DEC);
    if (DS18B20.getAddress(DS18B20_ADR,1))
        DS18B20.setResolution(DS18B20_ADR,DS18B20_RES);

    DS18B20.requestTemperaturesByAddress(DS18B20_ADR);
    
    Serial.print(F("TEMP: "));
    Serial.println(DS18B20.getTempC(DS18B20_ADR));

    Serial.print(F("Dusk: "));
    Serial.println(int(analogRead(Photo)));

    EEPROM.get(0,localDust);

    #ifdef DEBUG
        Serial.print(F("Dark: "));
        Serial.println(localDust.dark);
        Serial.print(F("LIGHT: "));
        Serial.println(localDust.light);
    #endif

    if (localDust.light > localDust.dark)
    {
        lightOn = localDust.dark + (localDust.light - localDust.dark)/3;
        lightOnMem = lightOn;
        lightOff = localDust.dark + (localDust.light - localDust.dark)*2/3;
    }

    #ifdef DEBUG
        Serial.print(F("Dark: "));
        Serial.println(localDust.dark);
        Serial.print(F("LIGHT: "));
        Serial.println(localDust.light);
    #endif

    // LED Ausgänge schalten
    pinMode(LED_G, OUTPUT);
    pinMode(LED_Y, OUTPUT);
    pinMode(LED_R, OUTPUT);
    pinMode(LED_W, OUTPUT);
    pinMode(LED_BUILTIN,OUTPUT);
    digitalWrite(LED_G, HIGH);
    digitalWrite(LED_Y, HIGH);
    digitalWrite(LED_R, HIGH);
    digitalWrite(LED_W, LOW);
    digitalWrite(LED_BUILTIN,LOW);

}

void loop()
{
    // Tastenfeldbeleuchtung mit Helligkeitssensor
    if ((int(analogRead(Photo)) < lightOn) and keyPad)
        digitalWrite(LED_W, HIGH);
    if (int(analogRead(Photo)) > lightOff) 
        digitalWrite(LED_W,LOW);

    Taste = Tastenfeld.getKey(); //Mit Unter der Variablen pressedKey entspricht der gedrückten Taste
    if (Taste and keyPad) //Wenn eine Taste gedrückt wurde...
    {
        #ifdef DEBUG
            Serial.println(Taste);
        #endif
        byte randNumber = random(0, 254);
        byte EnCodedKeyStroke = Taste ^ randNumber;
// Serial.println(EnCodedKeyStroke);
        com2Main.write(EnCodedKeyStroke);
        digitalWrite(LED_BUILTIN, !digitalRead(LED_BUILTIN));
    }

    if (com2Main.available())
    {
        byte res = com2Main.read();
        #ifdef DEBUG
            Serial.println(res);
        #endif
        if (res == 101)
        {
            digitalWrite(LED_G, HIGH);
            digitalWrite(LED_Y, HIGH);
            digitalWrite(LED_R, LOW);
        }
        if (res == 102)
        {
            digitalWrite(LED_G, LOW);
            digitalWrite(LED_Y, HIGH);
            digitalWrite(LED_R, HIGH);
        }
        if (res == 120)
        {
            digitalWrite(LED_G,HIGH);
            digitalWrite(LED_Y,LOW);
            digitalWrite(LED_R,LOW);
        }
        if (res == 130) // Fehler, ROT
        {
            keyPad = false;
            digitalWrite(LED_G,LOW);
            digitalWrite(LED_Y,LOW);
            digitalWrite(LED_R,HIGH);
        }
        if (res == 150) // Neuen Schlüssel empfangen
        {
            digitalWrite(LED_G, HIGH);
            digitalWrite(LED_Y, HIGH);
            digitalWrite(LED_R, HIGH);
        
            byte RecInititalKeyLength = 0;
            while (RecInititalKeyLength < 4)
            {
                byte inByte = 0;
                if (com2Main.available())
                {
                    inByte = com2Main.read();
                    initkey.as_array[RecInititalKeyLength] = inByte;
                    RecInititalKeyLength++;
                    #ifdef DEBUG
                        // Serial.print(RecInititalKeyLength);
                        // Serial.print(F(": "));
                        // Serial.println(inByte);
                    #endif
                }
            }
            Serial.println(initkey.as_long);
            randomSeed(initkey.as_long+presharedKey);
            digitalWrite(LED_G,LOW);
            digitalWrite(LED_Y,HIGH);
            digitalWrite(LED_R,LOW);
            keyPad = true;
        }
        if (res == 200) // Erwarte Eingabe, Gelb
        {
            keyPad = true;
            digitalWrite(LED_G,LOW);
            digitalWrite(LED_Y,HIGH);
            digitalWrite(LED_R,LOW);
        }
        if (res == 50)
        {
            DS18B20.requestTemperaturesByAddress(DS18B20_ADR);
            
            Serial.println(DS18B20.getTempC(DS18B20_ADR));
            DS18B20.requestTemperaturesByAddress(DS18B20_ADR);
            float temp = DS18B20.getTempC(DS18B20_ADR);
            #ifdef DEBUG
                // Serial.print(F("Temp: "));
                // Serial.println(temp);
            #endif
            com2Main.print(F("Temp OUT: "));
            com2Main.println(temp);
        }
        if (res == 55) //READ_LIGHT
        {
            #ifdef DEBUG
                // Serial.print(F("LIGTH: "));
                // Serial.println(int(analogRead(Photo)));
            #endif
            com2Main.print(F("LIGTH: "));
            com2Main.println(int(analogRead(Photo)));
        }
        if (res == 56) //LIGHT ON
        {
            lightOn = 1024;
            Serial.println(F("Ligth ON"));
            com2Main.println(F("Ligth ON"));
        }
        if (res == 57) //LIGHT OFF
        {
            lightOn = lightOnMem;
            Serial.println(F("Ligth OFF"));
            com2Main.println(F("Ligth OFF"));
        }
        if (res == 60)
        {
            // set light
            localDust.light =  int(analogRead(Photo));
            EEPROM.put(0,localDust);
            res = 61;
        }
        if (res == 61)
        {
            // get light
            com2Main.print(F("Light: "));
            com2Main.println(localDust.light);
        }
        if (res == 65)
        {
            // set dark
            localDust.dark =  int(analogRead(Photo));
            EEPROM.put(0,localDust);
            res = 66;
        }
        if (res == 66)
        {
            // get dark
            com2Main.print(F("Dark: "));
            com2Main.println(localDust.dark);
        }
        if (res == 250)
        {
            Serial.println(F("LOCK"));
            keyPad = false;
            digitalWrite(LED_G,LOW);
            digitalWrite(LED_Y,LOW);
            digitalWrite(LED_R,LOW);            
        } 
    }

}